import { DocumentNode, FragmentDefinitionNode, GraphQLSchema, OperationDefinitionNode } from "graphql";
export declare type State = {
    set: (key: string, value?: any) => any;
    get: (key: string) => any;
};
export declare type NetlifySite = {
    id: string;
};
export declare type NetlifyGraphConfig = {
    functionsPath: string[];
    netlifyGraphImplementationFilename: string[];
    netlifyGraphTypeDefinitionsFilename: string[];
    graphQLOperationsSourceFilename: string[];
    graphQLSchemaFilename: string[];
    netlifyGraphRequirePath: string[];
    netlifyGraphPath: string[];
    framework: string;
    extension: string;
    moduleType: "commonjs" | "esm";
};
export declare type ParsedFunction = {
    id: string;
    definition: string;
    description: string;
    fnName: string;
    safeBody: string;
    kind: "query" | "mutation" | "subscription";
    variableSignature: string;
    returnSignature: string;
    operationName: string;
    parsedOperation: OperationDefinitionNode;
};
export declare const defaultSourceOperationsFilename = "netlifyGraphOperationsLibrary.graphql";
export declare const defaultGraphQLSchemaFilename = "netlifyGraphSchema.graphql";
export declare const defaultNetlifyGraphConfig: NetlifyGraphConfig;
export declare const defaultExampleOperationsDoc = "query ExampleQuery @netlifyGraph(doc: \"An example query to start with.\") {\n  __typename\n}";
export declare const generateSubscriptionFunctionTypeDefinition: (schema: GraphQLSchema, fn: ParsedFunction, fragments: FragmentDefinitionNode[]) => string;
export declare const generateSubscriptionFunction: (schema: any, fn: any, fragments: any) => string;
export declare const queryToFunctionDefinition: (fullSchema: any, persistedQuery: any) => ParsedFunction | undefined;
export declare const generateJavaScriptClient: (netlifyGraphConfig: NetlifyGraphConfig, schema: GraphQLSchema, operationsDoc: string, enabledFunctions: ParsedFunction[]) => string;
export declare const generateTypeScriptDefinitions: (netlifyGraphConfig: NetlifyGraphConfig, schema: GraphQLSchema, enabledFunctions: ParsedFunction[]) => string;
export declare const generateFunctionsSource: (netlifyGraphConfig: NetlifyGraphConfig, schema: GraphQLSchema, operationsDoc: string, queries: any[]) => {
    clientSource: string;
    typeDefinitionsSource: string;
    functionDefinitions: ParsedFunction[];
};
export declare const extractFunctionsFromOperationDoc: (parsedDoc: DocumentNode) => {
    [k: string]: any;
};
/**
 * Given a schema, GraphQL operations doc, a target operationId, and a Netlify Graph config, generates a set of handlers (and potentially components) for the correct framework.
 */
export declare const generateHandlerSource: ({ handlerOptions, netlifyGraphConfig, operationId, operationsDoc, schema, }: {
    handlerOptions: Record<string, boolean>;
    netlifyGraphConfig: NetlifyGraphConfig;
    operationId: string;
    operationsDoc: string;
    schema: GraphQLSchema;
}) => {
    source: string;
    operation: OperationDefinitionNode;
} | undefined;
